
import ast
import sys
from weakref import WeakValueDictionary
import re
import os.path

# for self test
for i in range(10):
    ii = i

for i in range(10):
    if ii+1 == i:
        print(i)
        exit()

while i > 1000:
    exit()



def style():
    fname = 'style.css'
    if os.path.isfile(fname):
        return ''.join(open(fname).readlines())
    return '''
/* indetation */
.block {
    margin-left: 2em;
}

/*****************/
/* local styling */
.variable, .list, .condition {
    background: darkgreen; /*#CFC;*/
    color: white;
    font-weight: bold;
    
    padding: 0.1em .5em;
    border-radius: 2em;
}
.variable {
    background: white;
    color: darkgreen;
}
.kw {
    font-weight: bold;
}

/***********************/
/* funtion def styling */
.def {
    background: lightgrey;
    margin: 1em 0;
    padding: 0.5em;
    border: 2px solid black;
    border-radius: 5px;
}
.def>.block {
    background: white;
}
.statement.nocontinue {
    border-bottom-color: darkred;
    border-bottom-style: double;
    border-bottom-width: 8px;
}
.statement.return {
    border-bottom-color: black;
}

/******************/
/* blocks styling */
.base:first-of-type { /* line with: if, for, while */
    padding-bottom: .3em;
}
.if>.base~.block+.base { /* line with: else */
    padding-top: 0.4em;
    padding-bottom: 0.2em;
}
.group {	
    padding: .4em 0 1em 0.5em;
    
    border-width: 1px;
    border-style: solid;
    border-color: #303;
    border-radius:5px;
    
}
.statement {
    background: #DFD; /*#AFA !important;*/
    border-width: 1px;
    border-style: solid;
    border-color: darkgreen;
    padding: 5px;
    border-radius:5px;
}


/*****************/
/* nested blocks */
/*    a few alternating colors */
.root>.group,
.root>.group>.block>.group>.block>.group,
.root>.group>.block>.group>.block>.group>.block>.group>.block>.group,
.root>.group>.block>.group>.block>.group>.block>.group>.block>.group>.block>.group>.block>.group
 {
     background: #aaf;
 }
.root>.group>.block>.group,
.root>.group>.block>.group>.block>.group>.block>.group,
.root>.group>.block>.group>.block>.group>.block>.group>.block>.group>.block>.group,
.root>.group>.block>.group>.block>.group>.block>.group>.block>.group>.block>.group>.block>.group>.block>.group
 {
     background: #faa;
 }
/*    default for deeper ones */
.group { background: rgba(100, 100, 255, 0.33); }

/**********************/
/* small arrows below */
.group {
    display: block;
    position: relative;
}
.group::before {
    content: "";
    position: absolute;
    bottom: -6px;
    left: 100px;
    border-width: 6px 8px 0;
    border-style: solid;
    border-color: #303 transparent;
    display: block;
    width: 0;
    z-index: 10;
}
.group::after {
    content: "";
    position: absolute;
    bottom: -4px;
    left: 100px;
    border-width: 5px 8px 0;
    border-style: solid;
    border-color: #CAF transparent;
    display: block;
    width: 0;
    z-index: 10;
}
.statement {
    display: block;
    position: relative;
}
.statement:not(.nocontinue)::after {
    content: "";
    position: absolute;
    bottom: -5px;
    left: 100px;
    border-width: 6px 6px 0;
    border-style: solid;
    border-color: #DFD transparent;
    display: block;
    width: 0;
    z-index: 10;
}
.statement:not(.nocontinue)::before {
    content: "";
    position: absolute;
    bottom: -6px;
    left: 100px;
    border-width: 6px 6px 0;
    border-style: solid;
    border-color: darkgreen transparent;
    display: block;
    width: 0;
    z-index: 10;
}
'''

indent = 2
out = []
def tag(name, cls, content=None):
    global indent
    if content is None:
        out.append('{}<{} class="{}">'.format(indent*' ', name, cls))
        indent += 2
    else:
        out.append('{}<{} class="{}">{}</{}>'.format(indent*' ', name, cls, content, name))
def end(name):
    global indent
    indent -= 2
    out.append('{}</{}>'.format(indent*' ', name))
def out(content):
    global indent
    out.append('{}{}'.format(indent*' ', content))
def kw(k):
    cl = k.replace(',', 'comma').replace(':', 'colon').replace('(', 'paropen').replace(')', 'parclose').replace('.', 'dot')
    tag('span', 'kw kw'+cl, k)

def process_body(b, cls='block'):
    for i,a in enumerate(b):
        tag('div', cls)
        process_statement(a)
        end('div')

def process_arguments(args):
    for i,a in enumerate(args.args):
        if i != 0:
            kw(',')
        out.append(a.arg)
        
def process_statement(a):
    t = type(a)
    if t == ast.If:
        tag('div', 'group if')
        tag('div', 'base')
        kw('if')
        tag('tt', 'condition', text(a.test, ':'))
        kw(':')
        end('div')
        process_body(a.body)
        if len(a.orelse) != 0:
            tag('div', 'base')
            kw('else')
            end('div')
            process_body(a.orelse)
        end('div')
    elif t in [ast.For, ast.While]:
        tag('div', 'group loop '+t.__name__.lower())
        tag('div', 'base')
        if t == ast.For:
            kw('for')
            tag('tt', 'variable', text(a.target, 'for'))
            kw('in')
            tag('tt', 'list', text(a.iter, ':'))
            kw(':')
        else:
            kw('while')
            tag('tt', 'condition', text(a.test, ':'))
            kw(':')
        end('div')
        process_body(a.body)
        if len(a.orelse) > 0:
            print("UNHANDLED orelse in loops")
        end('div')
    elif t == ast.FunctionDef:
        tag('div', 'def')
        kw('def')
        tag('tt', 'name', a.name)
        kw('(')
        process_arguments(a.args)
        kw(')')
        kw(':')
        process_body(a.body, 'block root')
        end('div')
    else:
        # terminate = is_terminating(a)
        # more = ',shape="box"' if terminate else ''
        # print('{}[label="{}",tooltip="{}"{}]'.format(prefix, a.lineno, escaped_source(a), more))
        # link(prefix, linkto)
        # return [prefix]
        txt = text(a, 'full')
        if re.match(r'^return[( ]', txt):
            tag('tt', 'statement nocontinue return', txt)
        elif re.match(r'^exit *( *)', txt):
            tag('tt', 'statement nocontinue exit', txt)
        else:
            tag('tt', 'statement', txt)
            

prog_file = None
lines = None

parent_node = None
child_nodes = None
def next_sibling(c):
    p = parent_node[c]
    children = child_nodes[p]
    return children[children.index(c)+1]
def parse_parents(p):
    if type(p) == list:
        for c in p:
            parse_parents(c)
    else:
        child_nodes[p] = []
        for c in ast.iter_child_nodes(p):
            parent_node[c] = p
            child_nodes[p].append(c)
            parse_parents(c)

def text(a, *ext):
    l = lines[a.lineno-1][:-1]
    if 'full' in ext:
        return l[a.col_offset:]
    n = next_sibling(a)
    end = len(l) if n.lineno != a.lineno else n.col_offset
    raw = l[a.col_offset:end]
    raw = re.sub(r'#[^"]*$', '', raw) # comments
    if 'for' in ext:
        raw = re.sub(r'in *$', '', raw)
    if ':' in ext:
        raw = re.sub(r': *$', '', raw)
    raw = re.sub(r' *$', '', raw)
    return raw

def generate_html(_prog_file, content = None):
    global indent
    indent = 2
    global out
    out = []

    global prog_file
    prog_file = _prog_file
    global lines
    lines = open(prog_file).readlines() #if content is None else content.split('\n')

    global parent_node
    parent_node = dict()
    global child_nodes
    child_nodes = dict()

    prog = ''.join(lines)
    a = ast.parse(prog)
    parse_parents(a.body)

    res = '''
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <title>{}</title>
    <style>
{}
    </style>
  </head>
  <body>
'''.format(prog_file, style())
    process_body(a.body, cls='element root')
    res += '\n'.join(out)
    res += '''
  </body>
</html>'''
    return res


if __name__ == '__main__':
    p = sys.argv[1] if len(sys.argv) > 1 else sys.argv[0]
    o = re.sub(r'.py$', '.html', p)
    res = generate_html(p)
    with open(o, 'w') as f:
        f.write(res)
    print('firefox {}'.format(o))
